@@ -6,8 +6,10 @@ A simple utility to import something by its string name.
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
+from typing import Any
 
-def import_item(name):
+
+def import_item(name: str) -> Any:
     """Import and return ``bar`` given the string ``foo.bar``.
 
     Calling ``bar = import_item("foo.bar")`` is the functional equivalent of
@@ -23,8 +25,8 @@ def import_item(name):
     mod : module object
         The module that was imported.
     """
-    
-    parts = name.rsplit('.', 1)
+
+    parts = name.rsplit(".", 1)
     if len(parts) == 2:
         # called with 'foo.bar....'
         package, obj = parts
@@ -32,7 +34,7 @@ def import_item(name):
         try:
             pak = getattr(module, obj)
         except AttributeError as e:
-            raise ImportError('No module named %s' % obj) from e
+            raise ImportError("No module named %s" % obj) from e
         return pak
     else:
         # called with un-dotted string
