@@ -22,17 +22,14 @@ from typing import (
     Tuple,
     Optional,
     cast,
-    Sequence,
-    Mapping,
     Any,
     Union,
-    Callable,
-    Iterator,
     TypeVar,
 )
+from collections.abc import Sequence, Mapping, Callable, Iterator
 
 if sys.version_info < (3, 12):
-    from typing_extensions import Self
+    from typing import Self
 else:
     from typing import Self
 
@@ -102,7 +99,7 @@ class LSString(str):
 # print_lsstring = result_display.register(LSString)(print_lsstring)
 
 
-class SList(list):
+class SList(list[Any]):
     """List derivative with a special access attributes.
 
     These are normal lists, but with the special attributes:
@@ -187,9 +184,9 @@ class SList(list):
         else:
             pred = pattern
         if not prune:
-            return type(self)([el for el in self if pred(match_target(el))])
+            return type(self)([el for el in self if pred(match_target(el))])  # type: ignore [no-untyped-call]
         else:
-            return type(self)([el for el in self if not pred(match_target(el))])
+            return type(self)([el for el in self if not pred(match_target(el))])  # type: ignore [no-untyped-call]
 
     def fields(self, *fields: List[str]) -> List[List[str]]:
         """Collect whitespace-separated fields from string list
@@ -261,26 +258,10 @@ class SList(list):
         return type(self)([t[1] for t in dsu])
 
 
-# FIXME: We need to reimplement type specific displayhook and then add this
-# back as a custom printer. This should also be moved outside utils into the
-# core.
-
-# def print_slist(arg):
-#     """ Prettier (non-repr-like) and more informative printer for SList """
-#     print("SList (.p, .n, .l, .s, .grep(), .fields(), sort() available):")
-#     if hasattr(arg,  'hideonce') and arg.hideonce:
-#         arg.hideonce = False
-#         return
-#
-#     nlprint(arg)   # This was a nested list printer, now removed.
-#
-# print_slist = result_display.register(SList)(print_slist)
-
-
 def indent(instr: str, nspaces: int = 4, ntabs: int = 0, flatten: bool = False) -> str:
     """Indent a string a given number of spaces or tabstops.
 
-    indent(str,nspaces=4,ntabs=0) -> indent str by ntabs+nspaces.
+    indent(str, nspaces=4, ntabs=0) -> indent str by ntabs+nspaces.
 
     Parameters
     ----------
@@ -300,9 +281,7 @@ def indent(instr: str, nspaces: int = 4, ntabs: int = 0, flatten: bool = False)
     str : string indented by ntabs and nspaces.
 
     """
-    if instr is None:
-        return
-    ind = '\t'*ntabs+' '*nspaces
+    ind = "\t" * ntabs + " " * nspaces
     if flatten:
         pat = re.compile(r'^\s*', re.MULTILINE)
     else:
@@ -363,25 +342,6 @@ def marquee(txt: str = "", width: int = 78, mark: str = "*") -> str:
     return '%s %s %s' % (marks,txt,marks)
 
 
-ini_spaces_re = re.compile(r'^(\s+)')
-
-
-def num_ini_spaces(strng: str) -> int:
-    """Return the number of initial spaces in a string"""
-    warnings.warn(
-        "`num_ini_spaces` is Pending Deprecation since IPython 8.17."
-        "It is considered for removal in in future version. "
-        "Please open an issue if you believe it should be kept.",
-        stacklevel=2,
-        category=PendingDeprecationWarning,
-    )
-    ini_spaces = ini_spaces_re.match(strng)
-    if ini_spaces:
-        return ini_spaces.end()
-    else:
-        return 0
-
-
 def format_screen(strng: str) -> str:
     """Format a string for screen printing.
 
@@ -419,38 +379,6 @@ def dedent(text: str) -> str:
     return '\n'.join([first, rest])
 
 
-def wrap_paragraphs(text: str, ncols: int = 80) -> List[str]:
-    """Wrap multiple paragraphs to fit a specified width.
-
-    This is equivalent to textwrap.wrap, but with support for multiple
-    paragraphs, as separated by empty lines.
-
-    Returns
-    -------
-    list of complete paragraphs, wrapped to fill `ncols` columns.
-    """
-    warnings.warn(
-        "`wrap_paragraphs` is Pending Deprecation since IPython 8.17."
-        "It is considered for removal in in future version. "
-        "Please open an issue if you believe it should be kept.",
-        stacklevel=2,
-        category=PendingDeprecationWarning,
-    )
-    paragraph_re = re.compile(r'\n(\s*\n)+', re.MULTILINE)
-    text = dedent(text).strip()
-    paragraphs = paragraph_re.split(text)[::2] # every other entry is space
-    out_ps = []
-    indent_re = re.compile(r'\n\s+', re.MULTILINE)
-    for p in paragraphs:
-        # presume indentation that survives dedent is meaningful formatting,
-        # so don't fill unless text is flush.
-        if indent_re.search(p) is None:
-            # wrap paragraph
-            p = textwrap.fill(p, ncols)
-        out_ps.append(p)
-    return out_ps
-
-
 def strip_email_quotes(text: str) -> str:
     """Strip leading email quotation characters ('>').
 
@@ -501,26 +429,6 @@ def strip_email_quotes(text: str) -> str:
     return text
 
 
-def strip_ansi(source: str) -> str:
-    """
-    Remove ansi escape codes from text.
-
-    Parameters
-    ----------
-    source : str
-        Source to remove the ansi from
-    """
-    warnings.warn(
-        "`strip_ansi` is Pending Deprecation since IPython 8.17."
-        "It is considered for removal in in future version. "
-        "Please open an issue if you believe it should be kept.",
-        stacklevel=2,
-        category=PendingDeprecationWarning,
-    )
-
-    return re.sub(r'\033\[(\d|;)+?m', '', source)
-
-
 class EvalFormatter(Formatter):
     """A String Formatter that allows evaluation of simple expressions.
 
@@ -528,6 +436,10 @@ class EvalFormatter(Formatter):
     standard string formatting), so if slicing is required, you must explicitly
     create a slice.
 
+    Note that on Python 3.14+ this version interprets `[]` as indexing operator
+    so you need to use generators instead of list comprehensions, for example:
+    `list(i for i in range(10))`.
+
     This is to be used in templating cases, such as the parallel batch
     script templates, where simple arithmetic on arguments is useful.
 
@@ -544,7 +456,7 @@ class EvalFormatter(Formatter):
     """
 
     def get_field(self, name: str, args: Any, kwargs: Any) -> Tuple[Any, str]:
-        v = eval(name, kwargs)
+        v = eval(name, kwargs, kwargs)
         return v, name
 
 #XXX: As of Python 3.4, the format string parsing no longer splits on a colon
@@ -603,7 +515,7 @@ class FullEvalFormatter(Formatter):
 
                 # do any conversion on the resulting object
                 # type issue in typeshed, fined in https://github.com/python/typeshed/pull/11377
-                obj = self.convert_field(obj, conversion)  # type: ignore[arg-type]
+                obj = self.convert_field(obj, conversion)
 
                 # format the object and append to the result
                 result.append(self.format_field(obj, ''))
@@ -635,7 +547,7 @@ class DollarFormatter(FullEvalFormatter):
         r"(.*?)\$(\$?[\w\.]+)(?=([^']*'[^']*')*[^']*$)"
     )
 
-    def parse(self, fmt_string: str) -> Iterator[Tuple[Any, Any, Any, Any]]:  # type: ignore
+    def parse(self, fmt_string: str) -> Iterator[Tuple[Any, Any, Any, Any]]:
         for literal_txt, field_name, format_spec, conversion in Formatter.parse(
             self, fmt_string
         ):
@@ -704,153 +616,6 @@ def _get_or_default(mylist: List[T], i: int, default: T) -> T:
         return mylist[i]
 
 
-def compute_item_matrix(
-    items: List[str],
-    row_first: bool = False,
-    empty: Optional[str] = None,
-    *,
-    separator_size: int = 2,
-    displaywidth: int = 80,
-) -> Tuple[List[List[int]], Dict[str, int]]:
-    """Returns a nested list, and info to columnize items
-
-    Parameters
-    ----------
-    items
-        list of strings to columize
-    row_first : (default False)
-        Whether to compute columns for a row-first matrix instead of
-        column-first (default).
-    empty : (default None)
-        default value to fill list if needed
-    separator_size : int (default=2)
-        How much characters will be used as a separation between each columns.
-    displaywidth : int (default=80)
-        The width of the area onto which the columns should enter
-
-    Returns
-    -------
-    strings_matrix
-        nested list of string, the outer most list contains as many list as
-        rows, the innermost lists have each as many element as columns. If the
-        total number of elements in `items` does not equal the product of
-        rows*columns, the last element of some lists are filled with `None`.
-    dict_info
-        some info to make columnize easier:
-
-        num_columns
-          number of columns
-        max_rows
-          maximum number of rows (final number may be less)
-        column_widths
-          list of with of each columns
-        optimal_separator_width
-          best separator width between columns
-
-    Examples
-    --------
-    ::
-
-        In [1]: l = ['aaa','b','cc','d','eeeee','f','g','h','i','j','k','l']
-        In [2]: list, info = compute_item_matrix(l, displaywidth=12)
-        In [3]: list
-        Out[3]: [['aaa', 'f', 'k'], ['b', 'g', 'l'], ['cc', 'h', None], ['d', 'i', None], ['eeeee', 'j', None]]
-        In [4]: ideal = {'num_columns': 3, 'column_widths': [5, 1, 1], 'optimal_separator_width': 2, 'max_rows': 5}
-        In [5]: all((info[k] == ideal[k] for k in ideal.keys()))
-        Out[5]: True
-    """
-    warnings.warn(
-        "`compute_item_matrix` is Pending Deprecation since IPython 8.17."
-        "It is considered for removal in in future version. "
-        "Please open an issue if you believe it should be kept.",
-        stacklevel=2,
-        category=PendingDeprecationWarning,
-    )
-    info = _find_optimal(
-        list(map(len, items)),  # type: ignore[arg-type]
-        row_first,
-        separator_size=separator_size,
-        displaywidth=displaywidth,
-    )
-    nrow, ncol = info["max_rows"], info["num_columns"]
-    if row_first:
-        return (
-            [
-                [
-                    _get_or_default(
-                        items, r * ncol + c, default=empty
-                    )  # type:ignore[misc]
-                    for c in range(ncol)
-                ]
-                for r in range(nrow)
-            ],
-            info,
-        )
-    else:
-        return (
-            [
-                [
-                    _get_or_default(
-                        items, c * nrow + r, default=empty
-                    )  # type:ignore[misc]
-                    for c in range(ncol)
-                ]
-                for r in range(nrow)
-            ],
-            info,
-        )
-
-
-def columnize(
-    items: List[str],
-    row_first: bool = False,
-    separator: str = "  ",
-    displaywidth: int = 80,
-    spread: bool = False,
-) -> str:
-    """Transform a list of strings into a single string with columns.
-
-    Parameters
-    ----------
-    items : sequence of strings
-        The strings to process.
-    row_first : (default False)
-        Whether to compute columns for a row-first matrix instead of
-        column-first (default).
-    separator : str, optional [default is two spaces]
-        The string that separates columns.
-    displaywidth : int, optional [default is 80]
-        Width of the display in number of characters.
-
-    Returns
-    -------
-    The formatted string.
-    """
-    warnings.warn(
-        "`columnize` is Pending Deprecation since IPython 8.17."
-        "It is considered for removal in future versions. "
-        "Please open an issue if you believe it should be kept.",
-        stacklevel=2,
-        category=PendingDeprecationWarning,
-    )
-    if not items:
-        return "\n"
-    matrix: List[List[int]]
-    matrix, info = compute_item_matrix(
-        items,
-        row_first=row_first,
-        separator_size=len(separator),
-        displaywidth=displaywidth,
-    )
-    if spread:
-        separator = separator.ljust(int(info["optimal_separator_width"]))
-    fmatrix: List[filter[int]] = [filter(None, x) for x in matrix]
-    sjoin = lambda x: separator.join(
-        [y.ljust(w, " ") for y, w in zip(x, cast(List[int], info["column_widths"]))]
-    )
-    return "\n".join(map(sjoin, fmatrix)) + "\n"
-
-
 def get_text_list(
     list_: List[str], last_sep: str = " and ", sep: str = ", ", wrap_item_with: str = ""
 ) -> str:
